(define-module (ui ldap-users ajax)
    :use-module (alterator woo)
    :use-module (alterator ajax)
    :use-module (alterator effect)
    :export (init))

(define *user-parameter-list*
  '("sn" "givenname" "patronym" "homedirectory" "loginshell" "o" "title" "telephonenumber" "mobile" "user"))

(define *passwd-parameter-list*
  '("auto" "passwd_1" "passwd_2" "passwd_auto"))

(define (update-users . data)
  (let ((user (if (pair? data) (car data) (form-value "user"))))
    ;;TODO: what is it?
    (if (not (string-contains user ";"))
      (begin
        (form-update-visibility "update_message" #f)
        (ui-reset-password)
	(form-update-value "username" user)
	(form-update-value-list
	  *user-parameter-list*
	  (woo-read-first "/ldap-users" 'user user))
	(woo "email_reset" "/ldap-users" 'user user)
	(update-mail-list user)))))

(define (update-mail-list . data)
  (let ((user (if (pair? data) (car data) (form-value "user"))))
    (form-update-value "newemail" "")
    (form-update-enum "email" (woo "email_list" "/ldap-users" 'user user))))

(define (ui-init . data)
  (for-each
    (lambda(x) (form-update-value x ""))
    (append '("newusername" "username") *user-parameter-list*))
  (ui-reset-password)
  (form-update-enum "user" (woo-list "/ldap-users/avail_user"))
  (form-update-enum "loginshell" (woo-list "/ldap-users/avail_shell"))
  (form-update-visibility "update_message" #f)
  (form-update-enum "email" '()))

(define (ui-usersave)
  (let ((user (form-value "user")))
    (if (string? user)
      (catch/message
	(lambda()
	  (form-update-visibility "update_message" #f)
	  (apply woo-write "/ldap-users"
		 'user (form-value "user")
		 'language (form-value "language")
		 (form-value-list (append *passwd-parameter-list* *user-parameter-list*)))
	  (woo "email_commit" "/ldap-users" 'user user 'language (form-value "language"))
	  (update-users user)
	  (form-update-visibility "update_message" #t))))))

(define (ui-userdel)
  (let ((user (form-value "user")))
    (if (string? user)
      (catch/message
	(lambda()
	  (woo-delete "/ldap-users" 'user user 'language (form-value "language"))
	  (ui-init))))))

(define (ui-useradd)
  (let ((newusername (form-value "newusername")))
    (catch/message
      (lambda()
	(woo-new "/ldap-users" 'newusername newusername 'language (form-value "language"))
	(form-update-enum "user" (woo-list "/ldap-users/avail_users"))
	(form-update-value "user" newusername)
	(form-update-value "newusername" "")
	(update-users newusername)))))

(define (ui-emailadd)
  (catch/message
    (lambda()
      (let ((user (form-value "user"))
	    (newemail (form-value "newemail")))
	(cond ((and (string? user) (string? newemail))
	       (woo "email_add" "/ldap-users"
		    'user user
		    'newemail newemail
		    'language (form-value "language"))
	       (update-mail-list)))))))

(define (ui-emaildel)
  (catch/message
    (lambda()
      (let ((user (form-value "user"))
	    (email (form-value "email")))
	(cond ((and (string? user) (string? email))
	       (woo "email_del" "/ldap-users"
		    'user user
		    'email email
		    'language (form-value "language"))
	       (update-mail-list)))))))

(define (ui-generate)
  (form-update-value-list
    '("passwd_auto")
    (car (woo "generate" "/ldap-users"))))

(define (ui-reset-password)
  (form-update-value "passwd_1" "")
  (form-update-value "passwd_2" "")
  (form-update-value "auto" #f)
  (ui-generate)
  (update-effect))

(define (init)
    (ui-init)

    (effect-hide "accordion" "user" "")
    (effect-hide "userdelete" "user" "")
    (effect-hide "usersave" "user" "")

    (effect-show "passwd_1" "auto" #f)
    (effect-show "passwd_2" "auto" #f)
    (effect-show "passwd_auto" "auto" #t)
    (effect-show "generate_button" "auto" #t)
    (init-effect)

    (form-bind "generate_button" "click" ui-generate)
    (form-bind "emailadd" "click" ui-emailadd)
    (form-bind "emaildel" "click" ui-emaildel)
    (form-bind "usersave" "click" ui-usersave)
    (form-bind "useradd" "click" ui-useradd)
    (form-bind "userdelete" "click" ui-userdel)
    (form-bind "user" "change" update-users))
